from __future__ import absolute_import

# For convenience, re-export from sage.numerical.interactive_simplex_method
from sage.numerical.interactive_simplex_method import InteractiveLPProblem, InteractiveLPProblemStandardForm
from sage.numerical.interactive_simplex_method import LPDictionary, LPAbstractDictionary, LPRevisedDictionary

from sage_numerical_interactive_mip.interactive_milp_problem import InteractiveMILPProblem, InteractiveMILPProblemStandardForm
from sage_numerical_interactive_mip.clean_dictionary import LPCleanDictionary
from sage_numerical_interactive_mip.backends.abstract_backend_dictionary import LPAbstractBackendDictionary

# The following depend on solvers, so we import them lazily.
from sage.misc.lazy_import import lazy_import
lazy_import('sage_numerical_interactive_mip.backends.glpk_backend_dictionary', "LPGLPKBackendDictionary")
lazy_import('sage_numerical_interactive_mip.backends.coin_backend_dictionary', "LPCoinBackendDictionary")



r"""
Interactive Simplex Method floating-point helper class

This module provides a "clean" dictionary view on a dictionary with
floating-point numbers.  Cleaning means to change almost-zeros to exact
zeros, allowing the Interactive Simplex Method to recognize primal and
dual feasibility and to avoid pivoting on zero pivot elements.  By
extension, it also changes almost-integers to exact integers, for the
benefit of mixed-integer programming.
"""

from sage.numerical.interactive_simplex_method \
    import LPDictionary, LPAbstractDictionary
from sage.modules.all import vector
from sage.rings.all import ZZ
from sage.matrix.all import matrix
from sage.misc.functional import round

class LPCleanDictionary(LPAbstractDictionary):
    r"""
    Construct a clean dictionary for an LP problem from a dictionary.

    INPUT:

        - ``dictionary`` -- the dictionary to be cleaned

        - ``epsilon`` -- (default: 1e-9) the tolerance of the cleaning process

    OUTPUT:

       - a :class:`dictionary for an LP problem
       <LPCleanDictionary>`

    EXAMPLES:

    One needs an instance inherited from :class:`LPAbstractDictionary`
    to initialize this class:

    Setting up a backend dictionary as the input::

        sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
        sage: x = p.new_variable(nonnegative=True)
        sage: p.add_constraint(-x[0] + x[1] <= 2)
        sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
        sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
        sage: lp, basis = p.interactive_lp_problem()
        sage: d = lp.dictionary(*basis)
        sage: view(d)  # not tested

    Create the clean dictionary using the backend dictionary::

        sage: from sage_numerical_interactive_mip.clean_dictionary \
              import LPCleanDictionary
        sage: clean = LPCleanDictionary(d, epsilon=1e-5)
        sage: TestSuite(clean).run(skip=['_test_not_implemented_methods', '_test_pickling'])
        sage: clean
        LP problem dictionary (use typeset mode to see details)
        sage: view(clean.dictionary())  # not tested
    """
    def __init__(self, dictionary, epsilon=1e-9):
        r"""
        See :class:`LPGLPKBackendDictionary` for documentation.

        TESTS::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: view(d)  # not tested
            sage: clean = LPCleanDictionary(d, epsilon=1e-5)
            sage: clean
            LP problem dictionary (use typeset mode to see details)
            sage: view(clean.dictionary())  # not tested
        """
        super(LPCleanDictionary, self).__init__()
        self._dict = dictionary
        self._epsilon = epsilon

    def _round_(self, var):
        r"""
        Round a number based on the epsilon.

        INPUT:

            - ``var`` -- the variable to be rounded

        EXAMPLE::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean._round_(0.1)
            0.100000000000000
            sage: clean._round_(0.00001)
            0
        """
        nearest_int = ZZ(round(var))
        if abs(var - nearest_int) <= self._epsilon:
            var = nearest_int
        return var

    def leave(self, v):
        r"""
        Set ``v`` as the leaving variable for the associated dictionary.

        INPUT:

        - ``v`` -- a basic variable of the associated dictionary,
        can be given as a string, an actual variable, or an integer
        interpreted as the index of a variable

        EXAMPLE::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.basic_variables()
            (w_0, w_1)
            sage: clean.leave(clean.basic_variables()[0])
            sage: d.leaving()
            w_0
        """
        super(LPCleanDictionary, self).leave(v)
        self._dict.leave(v)

    def enter(self, v):
        r"""
        Set ``v`` as the entering variable for the associated dictionary.

        INPUT:

        - ``v`` -- a basic variable of the associated dictionary,
        can be given as a string, an actual variable, or an integer
        interpreted as the index of a variable

        EXAMPLE::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.nonbasic_variables()
            (m_0, m_1)
            sage: clean.enter(clean.nonbasic_variables()[0])
            sage: d._entering
            m_0
        """
        super(LPCleanDictionary, self).enter(v)
        self._dict.enter(v)

    def _round_all_(self, iterable):
        r"""
        Round all element in an iterable.

        INPUT:

            - ``iterable`` -- the list to tranverse through

        EXAMPLE::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: l = [0.01, 0.000001]
            sage: clean._round_all_(l)
            (0.0100000000000000, 0.000000000000000)
        """
        return vector([self._round_(e) for e in iterable])

    def row_coefficients(self, v):
        r"""
        Return the coefficients of a basic variable

        INPUT:

        - ``v`` -- a basic variable of ``self``, can be given as a string, an
          actual variable, or an integer interpreted as the index of a variable

        OUTPUT:

        - a vector of coefficients of a basic variable

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.row_coefficients(clean.basic_variables()[0])
            (-1, 1)
        """
        coefs = self._dict.row_coefficients(v)
        return self._round_all_(coefs)

    def constant_terms(self):
        r"""
        Return the constant terms of relations of ``self``.

        OUTPUT:

        - a vector.

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: d.constant_terms()
            (2.0, 3e-07)
            sage: clean.constant_terms()
            (2, 0)
        """
        terms = self._dict.constant_terms()
        return self._round_all_(terms)

    def objective_coefficients(self):
        r"""
        Return coefficients of the objective of ``self``.

        OUTPUT:

        - a vector

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.objective_coefficients()
            (-5.5, -2.1)
        """
        coefs = self._dict.objective_coefficients()
        return self._round_all_(coefs)

    def objective_name(self):
        return self._dict.objective_name()

    def objective_value(self):
        r"""
        Return the value of the objective value.

        OUTPUT:

        - a number

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.objective_value()
            0
        """
        val = self._dict.objective_value()
        return self._round_(val)

    def basic_variables(self):
        r"""
        Return the basic variables of the associated dictionary.

        OUTPUT:

        - a vector

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.basic_variables()
            (w_0, w_1)
        """
        return self._dict.basic_variables()

    def nonbasic_variables(self):
        r"""
        Return the non-basic variables of the associated dictionary.

        OUTPUT:

        - a vector

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean.nonbasic_variables()
            (m_0, m_1)
        """
        return self._dict.nonbasic_variables()

    def dictionary(self):
        r"""
        Return a cleaned regular LP dictionary of the associated dictionary.

        OUTPUT:

        - an :class:`LP dictionary <LPDictionary>`

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.clean_dictionary \
                  import LPCleanDictionary
            sage: p = MixedIntegerLinearProgram(names=['m'], solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(0.001 * x[0] + 0.0005 * x[1] <= 0.0000003)
            sage: p.set_objective(-5.5 * x[0] + -2.1 * x[1])
            sage: lp, basis = p.interactive_lp_problem()
            sage: d = lp.dictionary(*basis)
            sage: clean = LPCleanDictionary(d, epsilon=1e-4)
            sage: clean_dict = clean.dictionary()
            sage: view(clean_dict)  #not tested
        """
        old_leaving = self._dict._leaving
        rows = []
        for i, var in enumerate(self.basic_variables()):
            self.leave(self.basic_variables()[i])
            rows.append(self.leaving_coefficients())

        A = matrix(rows)
        D = LPDictionary(A,
                         self.constant_terms(),
                         self.objective_coefficients(),
                         self.objective_value(),
                         self.basic_variables(),
                         self.nonbasic_variables(),
                         self.objective_name())
        self.leave(old_leaving)
        D.enter(self._dict._entering)
        D.leave(old_leaving)
        return D



import sage.numerical.backends.glpk_backend as backend
from sage.numerical.backends.glpk_backend \
    import glp_bs, glp_nl, glp_nu
from sage.modules.all import vector
from sage_numerical_interactive_mip.backends.abstract_backend_dictionary \
    import LPAbstractBackendDictionary
from sage.numerical.interactive_simplex_method import variable

class LPGLPKBackendDictionary(LPAbstractBackendDictionary):
    r"""
    Construct a dictionary for an LP problem from an backend.

    INPUT:

        - ``backend`` -- the backend where the dictionary is
            constructed from

    OUTPUT:

       - a :class:`backend dictionary for an LP problem
       <LPGLPKBackendDictionary>`

    EXAMPLES:

    One needs an instance of :class:`GLPKBackend` to initialize
    this class::

        sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
              import LPGLPKBackendDictionary
        sage: p = MixedIntegerLinearProgram(maximization=True, solver="GLPK")
        sage: x = p.new_variable(nonnegative=True)
        sage: p.add_constraint(-x[0] + x[1] <= 2)
        sage: p.add_constraint(8 * x[0] + 2 * x[1] <= 17)
        sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
        sage: b = p.get_backend()
        sage: d = LPGLPKBackendDictionary(b)
        sage: d
        LP problem dictionary (use typeset mode to see details)
    """
    def __init__(self, backend):
        r"""
        See :class:`LPGLPKBackendDictionary` for documentation.

        TESTS::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(8 * x[0] + 2 * x[1] <= 17)
            sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
            sage: b = p.get_backend()
            sage: d = LPGLPKBackendDictionary(b)
            sage: TestSuite(d).run(skip=['_test_pickling'])

        An exception will be raised if the problem is not in standard form
        i.e. with <= constraints and >= 0 variable bounds::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(8 * x[0] + 2 * x[1], min=17)
            sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
            sage: b = p.get_backend()
            sage: d = LPGLPKBackendDictionary(b)
            Traceback (most recent call last):
            ...
            AttributeError: Problem constraints not in standard form.
        """
        super(LPGLPKBackendDictionary, self).__init__(backend)

    def basic_variables(self):
        r"""
        Return the basic variables of ``self``.

        OUTPUT:

        - a vector

        EXAMPLES:

        Setting up the problem::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(8 * x[0] + 2 * x[1] <= 17)
            sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0

        Use function in :class:`LPGLPKBackendDictionary`::

            sage: d = LPGLPKBackendDictionary(b)

        Use function in :class:`InteractiveLPProblem`::

            sage: lp, basis = p.interactive_lp_problem()
            sage: lpd = lp.dictionary(*basis)

        Compare results::

            sage: d.basic_variables()
            (x_0, x_1)
            sage: lpd.basic_variables()
            (x_0, x_1)
        """
        col_basics = tuple(
            self._x[i]
            for i in range(self._backend.ncols())
            if self._backend.get_col_stat(i) == glp_bs
        )
        row_basics = tuple(
            self._x[i + self._backend.ncols()]
            for i in range(self._backend.nrows())
            if self._backend.get_row_stat(i) == glp_bs
        )
        return vector(col_basics + row_basics)

    def constant_terms(self):
        r"""
        Return the constant terms of relations of ``self``.

        OUTPUT:

        - a vector.

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(8 * x[0] + 2 * x[1] <= 17)
            sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0
            sage: d = LPGLPKBackendDictionary(b)
            sage: d.constant_terms()
            (1.3, 3.3)
        """
        col_const = tuple(
            self._backend.get_variable_value(i)
            for i in range(self._backend.ncols())
            if self._backend.get_col_stat(i) == glp_bs
        )
        row_const = tuple(
            self._backend.row_bounds(i)[1] - self._backend.get_row_prim(i)
            for i in range(self._backend.nrows())
            if self._backend.get_row_stat(i) == glp_bs
        )
        return vector(col_const + row_const)

    def column_coefficients(self, v):
        r"""
        Return coefficients of a nonbasic variable.

        INPUT:

        - ``v`` -- a nonbasic variable of ``self``, can be given as a string, an
          actual variable, or an integer interpreted as the index of a variable

        OUTPUT:

        - a vector

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(x[0] + x[1] - 7*x[2] + x[3] <= 22)
            sage: p.add_constraint(x[1] + 2*x[2] - x[3] <= 13)
            sage: p.add_constraint(5*x[0] + x[2] <= 11)
            sage: p.set_objective(2*x[0] + 3*x[1] + 4*x[2] + 13*x[3])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0
            sage: d = LPGLPKBackendDictionary(b)
            sage: vars = d.nonbasic_variables()
            sage: vars
            (x_0, x_1, w_0, w_2)
            sage: d.enter(vars[0])
            sage: d.entering_coefficients()   # indirect doctest
            (5.0, 36.0, 26.0)
            sage: d.enter(vars[1])
            sage: d.entering_coefficients()   # indirect doctest
            (0.0, 1.0, 2.0)
        """
        if v is not None:
            v = variable(self.coordinate_ring(), v)
            if v not in self.nonbasic_variables():
                raise ValueError("variable must be nonbasic")
        index = tuple(self._x).index(v)

        # Reverse signs for auxiliary variables
        def reverse_sign_for_auxiliary(i_v):
            i, v = i_v
            return (i, v) if i < self._backend.nrows() else (i, -v)
        if index < self._backend.ncols():
            tab_col = map(reverse_sign_for_auxiliary,
                          zip(*self._backend.eval_tab_col(
                              index + self._backend.nrows())))
        else:
            tab_col = map(reverse_sign_for_auxiliary,
                          zip(*self._backend.eval_tab_col(
                              index - self._backend.ncols())))

        # Sort the coefficients so coefficients of
        # problem variables comes first
        l = [0] * (self._backend.nrows())
        for (i, v) in tab_col:
            if i < self._backend.nrows():
                symbol = self._x[i + self._backend.ncols()]
            else:
                symbol = self._x[i - self._backend.nrows()]
            pos = tuple(self.basic_variables()).index(symbol)
            l[pos] = v

        return vector(l)

    def row_coefficients(self, v):
        r"""
        Return coefficients of the basic variable ``v``.

        These are the coefficients with which nonbasic variables are subtracted
        in the relation for ``v``.

        INPUT:

        - ``v`` -- a basic variable of ``self``, can be given as a string, an
          actual variable, or an integer interpreted as the index of a variable

        OUTPUT:

        - a vector

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(x[0] + x[1] - 7*x[2] + x[3] <= 22)
            sage: p.add_constraint(x[1] + 2*x[2] - x[3] <= 13)
            sage: p.add_constraint(5*x[0] + x[2] <= 11)
            sage: p.set_objective(2*x[0] + 3*x[1] + 4*x[2] + 13*x[3])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0
            sage: d = LPGLPKBackendDictionary(b)
            sage: vars = d.basic_variables()
            sage: vars
            (x_2, x_3, w_1)
            sage: d.leave(vars[0])
            sage: d.leaving_coefficients()   # indirect doctest
            (5.0, 0.0, 0.0, 1.0)
            sage: d.leave(vars[1])
            sage: d.leaving_coefficients()   # indirect doctest
            (36.0, 1.0, 1.0, 7.0)
        """
        if v is not None:
            v = variable(self.coordinate_ring(), v)
            if v not in self.basic_variables():
                raise ValueError("variable must be basic")
        index = tuple(self._x).index(v)

        # Reverse signs for auxiliary variables
        def reverse_sign_for_auxiliary(i_v):
            i, v = i_v
            return (i, v) if i < self._backend.nrows() else (i, -v)
        def reverse_sign_for_nonauxiliary(i_v):
            i, v = i_v
            return (i, -v) if i < self._backend.nrows() else (i, v)
        if index < self._backend.ncols():
            raw_row = self._backend.eval_tab_row(
                index + self._backend.nrows())
            tab_row = map(reverse_sign_for_auxiliary, zip(*raw_row))
        else:
            raw_row = self._backend.eval_tab_row(
                index - self._backend.ncols())
            tab_row = map(reverse_sign_for_nonauxiliary, zip(*raw_row))

        l = [0] * (self._backend.ncols())
        for (i, v) in tab_row:
            if i < self._backend.nrows():
                symbol = self._x[i + self._backend.ncols()]
            else:
                symbol = self._x[i - self._backend.nrows()]
            pos = tuple(self.nonbasic_variables()).index(symbol)
            l[pos] = v

        return vector(l)

    def nonbasic_variables(self):
        r"""
        Return non-basic variables of ``self``.

        OUTPUT:

        - a vector

        EXAMPLES:

        Setting up the problem::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(8 * x[0] + 2 * x[1] <= 17)
            sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0

        Use function in :class:`LPGLPKBackendDictionary`::

            sage: d = LPGLPKBackendDictionary(b)

        Use function in :class:`InteractiveLPProblem`::

            sage: lp, basis = p.interactive_lp_problem()
            sage: lpd = lp.dictionary(*basis)

        Compare results:

            sage: d.nonbasic_variables()
            (w_0, w_1)
            sage: lpd.nonbasic_variables()
            (w_0, w_1)
        """
        col_nonbasics = tuple(
            self._x[i]
            for i in range(self._backend.ncols())
            if self._backend.get_col_stat(i) != glp_bs
        )
        row_nonbasics = tuple(
            self._x[i + self._backend.ncols()]
            for i in range(self._backend.nrows())
            if self._backend.get_row_stat(i) != glp_bs
        )
        return vector(col_nonbasics + row_nonbasics)

    def objective_coefficients(self):
        r"""
        Return coefficients of the objective of ``self``.

        OUTPUT:

        - a vector

        EXAMPLES:

        Setting up the problem::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(-x[0] + x[1] <= 2)
            sage: p.add_constraint(8 * x[0] + 2 * x[1] <= 17)
            sage: p.set_objective(5.5 * x[0] + 2.1 * x[1])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0

        Use function in :class:`LPGLPKBackendDictionary`::

            sage: d = LPGLPKBackendDictionary(b)

        Use function in :class:`InteractiveLPProblem`::

            sage: lp, basis = p.interactive_lp_problem()
            sage: lpd = lp.dictionary(*basis)

        Compare results::

            sage: d.objective_coefficients()   # rel tol 1e-9
            (-0.58, -0.76)
            sage: lpd.objective_coefficients() # rel tol 1e-9
            (-0.58, -0.76)
        """
        col_coefs = tuple(
            self._backend.get_col_dual(i)
            for i in range(self._backend.ncols())
            if self._backend.get_col_stat(i) != glp_bs
        )
        row_coefs = tuple(
            -self._backend.get_row_dual(i)
            for i in range(self._backend.nrows())
            if self._backend.get_row_stat(i) != glp_bs
        )
        return vector(col_coefs + row_coefs)

    def objective_name(self):
        r"""
        Return the objective name of ``self``.

        OUTPUT:

        - a symbolic expression

        """
        return SR("obj")

    def update(self):
        r"""
        Update ``self`` using previously set entering and leaving variables.

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(x[0] + x[1] - 7*x[2] + x[3] <= 22)
            sage: p.add_constraint(x[1] + 2*x[2] - x[3] <= 13)
            sage: p.add_constraint(5*x[0] + x[2] <= 11)
            sage: p.set_objective(2*x[0] + 3*x[1] + 4*x[2] + 13*x[3])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0
            sage: d = LPGLPKBackendDictionary(b)
            sage: d.objective_value()
            1331.0
            sage: d.nonbasic_variables()
            (x_0, x_1, w_0, w_2)
            sage: d.enter(d.nonbasic_variables()[0])
            sage: d.basic_variables()
            (x_2, x_3, w_1)
            sage: d.leave(d.basic_variables()[0])
            sage: d.objective_value()
            1331.0
            sage: d.update()
            sage: d.basic_variables()
            (x_0, x_3, w_1)
            sage: d.nonbasic_variables()
            (x_1, x_2, w_0, w_2)
            sage: d.objective_value()
            261.8

        TESTS:

        An error will be raised if the pivot selected is zero::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(x[0] + x[1] - 7*x[2] + x[3] <= 22)
            sage: p.add_constraint(x[1] + 2*x[2] - x[3] <= 13)
            sage: p.add_constraint(5*x[0] + x[2] <= 11)
            sage: p.set_objective(2*x[0] + 3*x[1] + 4*x[2] + 13*x[3])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0
            sage: d = LPGLPKBackendDictionary(b)
            sage: d.leave(d.basic_variables()[0])
            sage: d.leaving_coefficients()
            (5.0, 0.0, 0.0, 1.0)
            sage: d.enter(d.nonbasic_variables()[1])
            sage: d.leave(d.basic_variables()[0])
            sage: d.update()
            Traceback (most recent call last):
            ...
            ValueError: incompatible choice of entering and leaving variables
        """
        entering = self._entering
        if entering is None:
            raise ValueError("entering variable must be set before updating")

        leaving = self._leaving
        if leaving is None:
            raise ValueError("leaving variable must be set before updating")

        matching_index = tuple(self.basic_variables()).index(leaving)
        coef = self.entering_coefficients()[matching_index]
        if coef == 0:
            raise ValueError("incompatible choice of entering and leaving "
                             "variables")

        entering_index = tuple(self._x).index(entering)
        if entering_index < self._backend.ncols():
            self._backend.set_col_stat(entering_index, glp_bs)
        else:
            self._backend.set_row_stat(entering_index - self._backend.ncols(),
                                       glp_bs)

        leaving_index = tuple(self._x).index(leaving)
        if leaving_index < self._backend.ncols():
            self._backend.set_col_stat(leaving_index, glp_nl)
        else:
            self._backend.set_row_stat(leaving_index - self._backend.ncols(),
                                       glp_nu)

        if self._backend.warm_up() != 0:
            raise AttributeError("Warm up failed.")

    def add_row(self, nonbasic_coef, constant, slack_variable,
                integer_slack_variable=False):
        r"""
        Update a dictionary with an additional row based on a given dictionary.

        INPUT:

        - ``nonbasic_coef``-- a list of nonbasic coefficients for the new row

        - ``constant``-- a number of the constant term for the new row

        - ``slack_variable``-- a string of the name for the new slack variable

        - ``integer_slack_variable``-- (default: False) a boolean value
        indicating if the new slack variable is integer or not.

        EXAMPLES::

            sage: from sage_numerical_interactive_mip.backends.glpk_backend_dictionary \
                  import LPGLPKBackendDictionary
            sage: p = MixedIntegerLinearProgram(maximization=True, \
                                                solver="GLPK")
            sage: x = p.new_variable(nonnegative=True)
            sage: p.add_constraint(x[0] + x[1] - 7*x[2] + x[3] <= 22)
            sage: p.add_constraint(x[1] + 2*x[2] - x[3] <= 13)
            sage: p.add_constraint(5*x[0] + x[2] <= 11)
            sage: p.set_objective(2*x[0] + 3*x[1] + 4*x[2] + 13*x[3])
            sage: b = p.get_backend()
            sage: import sage.numerical.backends.glpk_backend as backend
            sage: b.solver_parameter(\
                backend.glp_simplex_or_intopt, backend.glp_simplex_only)
            sage: b.solve()
            0
            sage: d = LPGLPKBackendDictionary(b)
            sage: d.basic_variables()
            (x_2, x_3, w_1)
            sage: d.nonbasic_variables()
            (x_0, x_1, w_0, w_2)
            sage: d.objective_coefficients()
            (-486.0, -10.0, -13.0, -95.0)
            sage: d.add_row(range(3,7), 2, 'z_0')
            sage: d.objective_coefficients()
            (-486.0, -10.0, -13.0, -95.0)
            sage: d.basic_variables()
            (x_2, x_3, w_1, z_0)
            sage: d.leave(d.basic_variables()[3])
            sage: d.leaving_coefficients()
            (3.0, 4.0, 5.0, 6.0)
            sage: b.solve()
            0
            sage: d.basic_variables()
            (x_2, x_3, w_1, z_0)
            sage: d.nonbasic_variables()
            (x_0, x_1, w_0, w_2)

        Variables have 0 as their coefficient will not show up in the
        tableau:

            sage: d.add_row(range(-2, 2), 5, 'z_1')
            sage: d.get_backend().row(4)
            ([2, 1, 0], [-1.0, -1.0, -7.0])
        """
        if len(nonbasic_coef) != self._backend.ncols():
            raise ValueError("Length of nonbasic coefficients incompatible")

        # Convert to problem variable coefficients
        coef_pairs, constant = (
            self._nonbasic_coef_to_problem_coef_(nonbasic_coef, constant)
        )
        self._backend.add_linear_constraint(
            coef_pairs, None, constant, slack_variable
        )

        # Update buffered variables
        row_index = self._backend.nrows() - 1
        self._load_new_variable_(
            index=row_index,
            name=self._backend.row_name(row_index),
            auxiliary=True
        )

        # Update basis status in the backend
        self._backend.set_row_stat(self._backend.nrows() - 1, glp_bs)
        if self._backend.warm_up() != 0:
            raise AttributeError("Warm up failed.")



# -*- coding: utf-8 -*-
#
# documentation build configuration file,
# from sage_sample, which was in turn
# inspired by slabbe configuration file created sphinx-quickstart
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# General information about the project.
import six

project = u"sage-numerical-interactive-mip"
authors = copyright = u'2015-2016 Peijun Xiao, 2020 Matthias Koeppe'
package_name = 'sage_numerical_interactive_mip'
package_folder = "../../"

import sys
import os

from sage.env import SAGE_DOC_SRC, SAGE_DOC, SAGE_SRC

try:
    import sage.all
except ImportError:
    raise RuntimeError("to build the documentation you need to be inside a Sage shell (run first the command 'sage -sh' in a shell")



# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath(package_folder))
#sys.path.append(os.path.join(SAGE_SRC, "sage_setup", "docbuild", "ext"))


print("Using sys.path = {}".format(sys.path))

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    #'sage_autodoc',            ## Not available on conda-forge sage!
    #'sage_package.sphinx',
    'sphinx.ext.doctest',
    'sphinx.ext.coverage',
    'sphinx.ext.extlinks',
    'matplotlib.sphinxext.plot_directive',
    #'sphinxcontrib.bibtex'
]

### from Sage src/doc/common/conf.py
# This code is executed before each ".. PLOT::" directive in the Sphinx
# documentation. It defines a 'sphinx_plot' function that displays a Sage object
# through matplotlib, so that it will be displayed in the HTML doc.
plot_html_show_source_link = False
plot_pre_code = """
def sphinx_plot(graphics, **kwds):
    import matplotlib.image as mpimg
    from sage.misc.temporary_file import tmp_filename
    import matplotlib.pyplot as plt
    ## Option handling is taken from Graphics.save
    try:
        from sage.plot.multigraphics import GraphicsArray
    except ImportError:
        from sage.plot.graphics import GraphicsArray
    options = dict()
    if not isinstance(graphics, GraphicsArray):
        options.update(graphics.SHOW_OPTIONS)
        options.update(graphics._extra_kwds)
    options.update(kwds)
    dpi = options.pop('dpi', None)
    transparent = options.pop('transparent', None)
    fig_tight = options.pop('fig_tight', None)
    figsize = options.pop('figsize', None)
    ## figsize handling is taken from Graphics.matplotlib()
    if figsize is not None and not isinstance(figsize, (list, tuple)):
        # in this case, figsize is a number and should be positive
        try:
            figsize = float(figsize) # to pass to mpl
        except TypeError:
            raise TypeError("figsize should be a positive number, not {0}".format(figsize))
        if figsize > 0:
            default_width, default_height=rcParams['figure.figsize']
            figsize=(figsize, default_height*figsize/default_width)
        else:
            raise ValueError("figsize should be positive, not {0}".format(figsize))

    if figsize is not None:
        # then the figsize should be two positive numbers
        if len(figsize) != 2:
            raise ValueError("figsize should be a positive number "
                             "or a list of two positive numbers, not {0}".format(figsize))
        figsize = (float(figsize[0]),float(figsize[1])) # floats for mpl
        if not (figsize[0] > 0 and figsize[1] > 0):
            raise ValueError("figsize should be positive numbers, "
                             "not {0} and {1}".format(figsize[0],figsize[1]))

    plt.figure(figsize=figsize)
    if isinstance(graphics, GraphicsArray):
        ## from GraphicsArray.save
        figure = plt.gcf()
        rows = graphics.nrows()
        cols = graphics.ncols()
        for i, g in enumerate(graphics):
            subplot = figure.add_subplot(rows, cols, i + 1)
            g_options = copy(options)
            g_options.update(g.SHOW_OPTIONS)
            g_options.update(g._extra_kwds)
            g_options.pop('dpi', None)
            g_options.pop('transparent', None)
            g_options.pop('fig_tight', None)
            g.matplotlib(figure=figure, sub=subplot, **g_options)
    else:
        figure = graphics.matplotlib(figure=plt.gcf(), figsize=figsize, **options)
    plt.tight_layout(pad=0)
    plt.margins(0)
    plt.show()

from sage.all_cmdline import *
"""

plot_html_show_formats = False
plot_formats = ['svg', 'pdf', 'png']

# Add any paths that contain templates here, relative to this directory.
# templates_path = ['_templates']
templates_path = [os.path.join(SAGE_DOC_SRC, 'common', 'templates'), '_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'



# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
from pkg_resources import get_distribution, DistributionNotFound
# The full version, including alpha/beta/rc tags.
try:
    release = get_distribution('sage-numerical-interactive-mip').version
except DistributionNotFound:
    release = "0.2"
#print("############# release reported: {} ##################".format(release))
# The short X.Y version.
version = '.'.join(release.split('.')[:2])

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all
# documents.
default_role = 'math'

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sage'
html_theme_path = ['../themes']

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {}

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []   #['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = package_name + "doc"


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
# The paper size ('letterpaper' or 'a4paper').
#'papersize': 'letterpaper',

# The font size ('10pt', '11pt' or '12pt').
#'pointsize': '10pt',

# Additional stuff for the LaTeX preamble.
'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  ('index', package_name + '.tex', u'Documentation of ' + six.text_type(package_name),
   authors, 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', package_name, six.text_type(package_name) + u" documentation",
     [authors], 1)
]

# If true, show URL addresses after external links.
#man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  ('index', package_name, six.text_type(package_name) + u" documentation",
   authors, package_name, project,
   'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#texinfo_appendices = []

# If false, no module index is generated.
#texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#texinfo_no_detailmenu = False

# -- Options copied from Sagemath conf.py file -------------------------------

# We use MathJax to build the documentation unless the environment
# variable SAGE_DOC_MATHJAX is set to "no" or "False".  (Note that if
# the user does not set this variable, then the script sage-env sets
# it to "True".)

if (os.environ.get('SAGE_DOC_MATHJAX', 'no') != 'no'
            and os.environ.get('SAGE_DOC_MATHJAX', 'no') != 'False'):

    extensions.append('sphinx.ext.mathjax')
    mathjax_path = 'MathJax.js?config=TeX-AMS_HTML-full,../mathjax_sage.js'

    from sage.misc.latex_macros import sage_mathjax_macros
    # this is broken for now
    # html_theme_options['mathjax_macros'] = sage_mathjax_macros()

    ## from pkg_resources import Requirement, working_set
    ## sagenb_path = working_set.find(Requirement.parse('sagenb')).location
    ## mathjax_relative = os.path.join('sagenb','data','mathjax')

    ## # It would be really nice if sphinx would copy the entire mathjax directory,
    ## # (so we could have a _static/mathjax directory), rather than the contents of the directory

    ## mathjax_static = os.path.join(sagenb_path, mathjax_relative)
    ## html_static_path.append(mathjax_static)
    ## exclude_patterns=['**/'+os.path.join(mathjax_relative, i) for i in ('docs', 'README*', 'test',
    ##                                                                     'unpacked', 'LICENSE')]
    from sage.env import SAGE_LOCAL, SAGE_SHARE
    html_static_path.append(SAGE_LOCAL + "/lib/mathjax")    # conda
    html_static_path.append(SAGE_SHARE + "/mathjax")  # sage distribution
else:
     extensions.append('sphinx.ext.pngmath')

# This is to make the verbatim font smaller;
# Verbatim environment is not breaking long lines
from sphinx.highlighting import PygmentsBridge
from pygments.formatters.latex import LatexFormatter

class CustomLatexFormatter(LatexFormatter):
    def __init__(self, **options):
        super(CustomLatexFormatter, self).__init__(**options)
        self.verboptions = r"formatcom=\footnotesize"

PygmentsBridge.latex_formatter = CustomLatexFormatter

latex_elements['preamble'] += r'''
% One-column index
\makeatletter
\renewenvironment{theindex}{
  \chapter*{\indexname}
  \markboth{\MakeUppercase\indexname}{\MakeUppercase\indexname}
  \setlength{\parskip}{0.1em}
  \relax
  \let\item\@idxitem
}{}
\makeatother
\renewcommand{\ttdefault}{txtt}
'''

#####################################################
# add LaTeX macros for Sage

from sage.misc.latex_macros import sage_latex_macros

try:
    pngmath_latex_preamble  # check whether this is already defined
except NameError:
    pngmath_latex_preamble = ""

for macro in sage_latex_macros():
    # used when building latex and pdf versions
    latex_elements['preamble'] += macro + '\n'
    # used when building html version
    pngmath_latex_preamble += macro + '\n'


## The following is needed on conda-forge sagemath
from sage.repl.user_globals import initialize_globals
import sage.all
my_globs = dict()
initialize_globals(sage.all, my_globs)
